using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.IO;
using System.Runtime.InteropServices;
using System.Windows.Forms;
using System.Globalization;
using Microsoft.VisualBasic.FileIO;

namespace aocrecs2
{
    public partial class frmMain : Form
    {
        private string aocpath;
        private string current_dir;
        ListViewItemComparer listViewItemSorter = new ListViewItemComparer();
        private List<Recgame> recs = new List<Recgame>();

        private Bitmap _bmp;
        private string original_name = "";
        private string datefmt;

        //clipboard
        private IDataObject iData;
        private MemoryStream copy_ms;
        private MemoryStream cut_ms;

        private Color[] player_color = { Color.FromArgb(0x00,0x00, 0xC0), Color.FromArgb(0xE0,0x00,0x00), 
                                         Color.FromArgb(0x00, 0xC0, 0x00), Color.FromArgb(0xC0, 0xC0, 0x00), 
                                         Color.FromArgb(0x00, 0xC0, 0xC0), Color.FromArgb(0xC0, 0x00, 0x80),   
                                         Color.FromArgb(0x80, 0x80, 0x80), Color.FromArgb(0xFF, 0x80, 0x00)};
        private Color[] player_point_color = { Color.FromArgb(0x00,0x00, 0xE0), Color.FromArgb(0xE0,0x00,0x00), 
                                             Color.FromArgb(0x00, 0xE0, 0x00), Color.FromArgb(0xE0, 0xE0, 0x00), 
                                             Color.FromArgb(0x00, 0xE0, 0xE0), Color.FromArgb(0xE0, 0x30, 0x90),   
                                             Color.FromArgb(0xC0, 0xC0, 0xC0), Color.FromArgb(0xFF, 0x80, 0x00)};


        private string[] player_civ =   { "", "Bri", "Fra", "Got", "Teu", "Jap", "Chi", "Byz",
                                          "Per", "Sar", "Tur", "Vik", "Mon", "Cel", "Spe", "Azt", 
                                          "May", "Hun", "Kor"};

        //----------------------------------
        // ICON
        //----------------------------------
        [DllImport("shell32.dll")]
        private static extern IntPtr SHGetFileInfo(string pszPath, uint dwFileAttributes, 
                                                   ref SHFILEINFO psfi, uint cbSizeFileInfo, uint uFlags);

        private const uint SHGFI_ICON = 0x100;
        private const uint SHGFI_LARGEICON = 0x0;
        private const uint SHGFI_SMALLICON = 0x1;

        private struct SHFILEINFO
        {
            public IntPtr hIcon;
            public IntPtr iIcon;
            public uint dwAttributes;
            [MarshalAs(UnmanagedType.ByValTStr, SizeConst = 260)]
            public string szDisplayName;
            [MarshalAs(UnmanagedType.ByValTStr, SizeConst = 80)]
            public string szTypeName;
        };

        public frmMain()
        {
            InitializeComponent();
        }

        private void Form1_Load(object sender, EventArgs e)
        {
            datefmt = DateTimeFormatInfo.CurrentInfo.ShortDatePattern + " HH:mm";
            lvwFile.Focus();

            //Init Clipboard
            copy_ms = new MemoryStream(BitConverter.GetBytes((int)DragDropEffects.Copy));
            cut_ms = new MemoryStream(BitConverter.GetBytes((int)DragDropEffects.Move));

            aocpath = getAOCRegistry();
            current_dir = aocpath;
            listViewItemSorter.Mode = ListViewItemComparer.ComparerMode.String;

            fileSystemWatcher1.Path = current_dir;
            showlist(aocpath);

            //Cxgnh̒ǉ
            fileSystemWatcher1.Changed += new System.IO.FileSystemEventHandler(watcher_Changed);
            fileSystemWatcher1.Created += new System.IO.FileSystemEventHandler(watcher_Changed);
            fileSystemWatcher1.Deleted += new System.IO.FileSystemEventHandler(watcher_Changed);
            fileSystemWatcher1.Renamed += new System.IO.RenamedEventHandler(watcher_Renamed);
        }

        private void picMap_Paint(object sender, PaintEventArgs e)
        {
            if (_bmp != null)
            {
                e.Graphics.DrawImage(_bmp, 0, 0, _bmp.Width, _bmp.Height);
            }
        }

        private void txtPath_KeyDown(object sender, KeyEventArgs e)
        {
            if (e.KeyCode == Keys.Enter)
            {
                showlist(txtPath.Text);
            }
        }

        private void txtPath_KeyPress(object sender, KeyPressEventArgs e)
        {
            if (e.KeyChar == (char)Keys.Enter)
            {
                e.Handled = true; //avoid beep
            }
        }

        private void btnPath_Click(object sender, EventArgs e)
        {
            FolderBrowserDialog fbd = new FolderBrowserDialog();

            fbd.Description = Properties.Resources.MSG_SELECT_FOLDER;
            fbd.RootFolder = Environment.SpecialFolder.Desktop;
            fbd.SelectedPath = current_dir;
            fbd.ShowNewFolderButton = false;

            if (fbd.ShowDialog(this) == DialogResult.OK)
            {
                current_dir = fbd.SelectedPath;
                txtPath.Text = fbd.SelectedPath;
                showlist(txtPath.Text);
            }
        }

        private void lvwFile_SelectedIndexChanged(object sender, EventArgs e)
        {
            Recgame rec;
            int display_width = 220;

            if (lvwFile.SelectedItems.Count > 0)
            {
                string filename = lvwFile.SelectedItems[0].SubItems[0].Text;

                if (filename.EndsWith(".mgx") != true)
                {
                    //Game Data
                    setData(null);

                    _bmp = new Bitmap(display_width, (int)(display_width / 2));

                    Graphics g1 = Graphics.FromImage(_bmp);
                    g1.Clear(picMap.BackColor);
                    g1.Dispose();

                    picMap.Invalidate();

                    return;
                }

                int index = (int)lvwFile.SelectedItems[0].Tag;

                rec = recs[index];

                if (rec.rec_load_flg == false)
                {
                    rec.file_name = filename;

                    int ret = rec.readRecgame(Path.Combine(current_dir, filename));
                    if (ret != 0) //ng
                    {
                        return;
                    }
                    lvwFile.SelectedItems[0].SubItems[1].Text = rec.players[rec.rec_player].name;

                    lvwFile.SelectedItems[0].SubItems[2].Text = rec.getMapName(rec.map_id);
                    lvwFile.SelectedItems[0].SubItems[3].Text = rec.vs;
                    lvwFile.SelectedItems[0].SubItems[4].Text = string.Format("{0:D}:{1:D2}:{2:D2}",
                                                                              rec.duration.Hours, 
                                                                              rec.duration.Minutes, 
                                                                              rec.duration.Seconds);
                }

                //Game Data
                setData(rec);

                //Map
                create_map(rec, rec.map_img, rec.mapsize_x, 220);

                picMap.Invalidate();

                //_bmp.MakeTransparent(picMap.BackColor);
                //_bmp.Save(rec.file_name.Substring(0, rec.file_name.Length - 3) + "png", System.Drawing.Imaging.ImageFormat.Png);
            }
        }

        private void create_map(Recgame rec, Bitmap src_img, int mapsize, int image_width)
        {
            Single scale = ((Single)(image_width / Math.Sqrt(2)) / mapsize);

            _bmp = new Bitmap(image_width, (int)(image_width / 2));

            Graphics g = Graphics.FromImage(_bmp);
            g.Clear(picMap.BackColor);

            g.RotateTransform(-45f);
            g.TranslateTransform(0, (Single)(mapsize / 2 * Math.Sqrt(2)), System.Drawing.Drawing2D.MatrixOrder.Append);
            g.ScaleTransform(scale, scale / 2, System.Drawing.Drawing2D.MatrixOrder.Append);
            g.DrawImage(src_img, new Point(0, 0));
            g.ResetTransform();

            double cos45 = Math.Cos((Math.PI / 180) * -45);
            double sin45 = Math.Sin((Math.PI / 180) * -45);

            //Town center
            for (int i = 1; i < rec.num_player; i++)
            {
                double x = (rec.players[i].pos_x * cos45 - rec.players[i].pos_y * sin45) * scale - 1;
                double y = (rec.players[i].pos_x * sin45 + rec.players[i].pos_y * cos45
                            + rec.mapsize_x / 2 * Math.Sqrt(2)) * scale / 2 - 1;

                g.FillRectangle(new SolidBrush(player_point_color[rec.players[i].color]), (Single)x, (Single)y, 3, 3);
            }

            g.Dispose();
        }


        private void lvwFile_DoubleClick(object sender, EventArgs e)
        {
            if (lvwFile.SelectedItems.Count > 0)
            {
                string fullpath = Path.Combine(current_dir, lvwFile.SelectedItems[0].SubItems[0].Text);
                FileAttributes fa = File.GetAttributes(fullpath);

                if ((fa & FileAttributes.Directory) == FileAttributes.Directory)
                {
                    showlist(fullpath);
                }
            }
        }

        private void lvwFile_KeyDown(object sender, KeyEventArgs e)
        {
            if (e.KeyCode == Keys.F2 && lvwFile.FocusedItem != null)
            {
                lvwFile.FocusedItem.BeginEdit();
            }
            else if (e.KeyCode == Keys.Enter)
            {
                if (lvwFile.SelectedItems.Count > 0)
                {
                    string fullpath = Path.Combine(current_dir, lvwFile.SelectedItems[0].SubItems[0].Text);
                    FileAttributes fa = File.GetAttributes(fullpath);

                    if ((fa & FileAttributes.Directory) == FileAttributes.Directory)
                    {
                        showlist(fullpath);
                    }
                }
            }
            else if (e.KeyCode == Keys.Escape)
            {
                //backgroundWorker1.CancelAsync();
            }
        }

        private void lvwFile_BeforeLabelEdit(object sender, LabelEditEventArgs e)
        {
            original_name = lvwFile.Items[e.Item].Text;
        }

        private void lvwFile_AfterLabelEdit(object sender, LabelEditEventArgs e)
        {
            if (e.Label != null)
            {
                try
                {
                    string fullpath = Path.Combine(current_dir, original_name);
                    FileAttributes fa = File.GetAttributes(fullpath);

                    if ((fa & FileAttributes.Directory) == FileAttributes.Directory)
                    {
                        Directory.Move(Path.Combine(current_dir, original_name), Path.Combine(current_dir, e.Label));
                    }
                    else 
                    { 
                        File.Move(Path.Combine(current_dir, original_name), Path.Combine(current_dir,e.Label));                  
                    }
                }
                catch(Exception)
                {
                    e.CancelEdit = true;
                }
            }

        }

        private void lvwFile_ColumnClick(object sender, ColumnClickEventArgs e)
        {
            listViewItemSorter.Column = e.Column;
            switch (e.Column)
            {
                case 4:
                    listViewItemSorter.Mode = ListViewItemComparer.ComparerMode.DateTime;
                    listViewItemSorter.Dateformat = "H:mm:ss";
                    break;
                case 5:
                    listViewItemSorter.Mode = ListViewItemComparer.ComparerMode.DateTime;
                    listViewItemSorter.Dateformat = datefmt;
                    break;
                default:
                    listViewItemSorter.Mode = ListViewItemComparer.ComparerMode.String;
                    break;

            }

            lvwFile.Sort();
        }

        private void lvwFile_DragOver(object sender, DragEventArgs e)
        {
            if (e.Data.GetDataPresent(DataFormats.FileDrop))
            {
                if ((e.KeyState & 8) == 8) // 4:shift, 8:ctrl, 32:alt
                {
                    e.Effect = DragDropEffects.Copy;
                }
                else
                {
                    e.Effect = DragDropEffects.Move;
                }
            }
            else
            {
                e.Effect = DragDropEffects.None;
            }
        }

        private void lvwFile_DragDrop(object sender, DragEventArgs e)
        {
            if (e.Data.GetDataPresent(DataFormats.FileDrop))
            {
                string[] fileNames = (string[])e.Data.GetData(DataFormats.FileDrop, false);

                foreach (string fileName in fileNames)
                {
                    FileAttributes fa = File.GetAttributes(fileName);

                    if ((fa & FileAttributes.Directory) == FileAttributes.Directory)
                    {
                        if ((e.KeyState & 8) == 8) // 4:shift, 8:ctrl, 32:alt
                        {
                            FileSystem.CopyDirectory(fileName, Path.Combine(current_dir, Path.GetFileName(fileName)),
                                                     UIOption.AllDialogs, UICancelOption.DoNothing);
                        }
                        else
                        {
                            Directory.Move(fileName, Path.Combine(current_dir, Path.GetFileName(fileName)));
                        }
                    }
                    else
                    {
                        if ((e.KeyState & 8) == 8) // 4:shift, 8:ctrl, 32:alt
                        {
                            File.Copy(fileName, Path.Combine(current_dir, Path.GetFileName(fileName)));
                        }
                        else
                        {
                            File.Move(fileName, Path.Combine(current_dir, Path.GetFileName(fileName)));
                        }
                    }
                }
            }
        }

        private void lvwFile_ItemDrag(object sender, ItemDragEventArgs e)
        {
            string[] filenames = new string[lvwFile.SelectedItems.Count];

            for (int i = 0; i < lvwFile.SelectedItems.Count; i++)
            {
                filenames[i] = Path.Combine(current_dir, lvwFile.SelectedItems[i].Text);
            }

            DataObject dataObj = new DataObject(DataFormats.FileDrop, filenames);
            DragDropEffects effect = DragDropEffects.Copy | DragDropEffects.Move;
            lvwFile.DoDragDrop(dataObj, effect);
        }

        //----------------------------------
        // Menu
        //----------------------------------
        private void MenuItemExit_Click(object sender, EventArgs e)
        {
            Application.Exit();
        }

        private void MenuItemEdit_DropDownOpening(object sender, EventArgs e)
        {
            if (this.ActiveControl == lvwFile)
            {
                if (lvwFile.SelectedItems.Count > 0)
                {
                    MenuItemCut.Enabled = true;
                    MenuItemCopy.Enabled = true;
                    MenuItemDelete.Enabled = true;
                    MenuItemRename.Enabled = true;
                }
                else
                {
                    MenuItemCut.Enabled = false;
                    MenuItemCopy.Enabled = false;
                    MenuItemDelete.Enabled = false;
                    MenuItemRename.Enabled = false;
                }
                if (txtNameP1.Text != "")
                {
                    MenuItemMapCopy.Enabled = true;
                }
                else
                {
                    MenuItemMapCopy.Enabled = false;
                }
                if (Clipboard.ContainsFileDropList() == true)
                {
                    MenuItemPaste.Enabled = true;
                }
                else
                {
                    MenuItemPaste.Enabled = false;
                }

            }
            else if (this.ActiveControl is TextBox)
            {
                TextBox t = (TextBox)this.ActiveControl;
                if (t.SelectionLength > 0)
                {
                    MenuItemCut.Enabled = true;
                    MenuItemCopy.Enabled = true;
                    MenuItemMapCopy.Enabled = false;
                    MenuItemDelete.Enabled = false;
                    MenuItemRename.Enabled = false;
                }
                else
                {
                    MenuItemCut.Enabled = false;
                    MenuItemCopy.Enabled = false;
                    MenuItemMapCopy.Enabled = false;
                    MenuItemDelete.Enabled = false;
                    MenuItemRename.Enabled = false;
                }
                if (Clipboard.ContainsText() == true)
                {
                    MenuItemPaste.Enabled = true;
                }
                else
                {
                    MenuItemPaste.Enabled = false;
                }
            }
            else
            {
                MenuItemMapCopy.Enabled = false;
                MenuItemCut.Enabled = false;
                MenuItemCopy.Enabled = false;
                MenuItemPaste.Enabled = false;
                MenuItemDelete.Enabled = false;
                MenuItemRename.Enabled = false;
            }
        }

        private void MenuItemCut_Click(object sender, EventArgs e)
        {
            if (this.ActiveControl == lvwFile)
            {
                FileCopyPaste("cut");
            }
            else if (this.ActiveControl is TextBox)
            {
                TextBox t = (TextBox)this.ActiveControl;
                if (t.ReadOnly == false)
                {
                    t.Cut();
                }
                else
                {
                    t.Copy();
                }

            }
        }

        private void MenuItemCopy_Click(object sender, EventArgs e)
        {
            if (this.ActiveControl == lvwFile)
            {
                FileCopyPaste("copy");
            }
            else if (this.ActiveControl is TextBox)
            {
                TextBox t = (TextBox)this.ActiveControl;
                t.Copy();
            }
        }

        private void MenuItemPaste_Click(object sender, EventArgs e)
        {
            if (this.ActiveControl == lvwFile)
            {
                FileCopyPaste("paste");
            }
            else if (this.ActiveControl is TextBox)
            {
                TextBox t = (TextBox)this.ActiveControl;
                t.Paste();
            }
        }

        private void MenuItemDelete_Click(object sender, EventArgs e)
        {
            if (this.ActiveControl == lvwFile)
            {
                FileCopyPaste("delete");
            }
        }

        private void MenuItemMapCopy_Click(object sender, EventArgs e)
        {
            if (_bmp != null)
            {
                Clipboard.SetDataObject(_bmp, true);
            }
        }

        private void cMenuStrip1_Opening(object sender, CancelEventArgs e)
        {
            if (lvwFile.SelectedItems.Count > 0)
            {
                cMenuCut.Enabled = true;
                cMenuCopy.Enabled = true;
                cMenuDelete.Enabled = true;
            }
            else
            {
                cMenuCut.Enabled = false;
                cMenuCopy.Enabled = false;
                cMenuDelete.Enabled = false;
            }
            if (Clipboard.ContainsFileDropList() == true)
            {
                cMenuPaste.Enabled = true;
            }
            else
            {
                cMenuPaste.Enabled = false;
            }
        }

        private void cMenuCut_Click(object sender, EventArgs e)
        {
            if (this.ActiveControl == lvwFile)
            {
                FileCopyPaste("cut");
            }
        }

        private void cMenuCopy_Click(object sender, EventArgs e)
        {
            if (this.ActiveControl == lvwFile)
            {
                FileCopyPaste("copy");
            }
        }

        private void cMenuPaste_Click(object sender, EventArgs e)
        {
            if (this.ActiveControl == lvwFile)
            {
                FileCopyPaste("paste");
            }
        }

        private void cMenuDelete_Click(object sender, EventArgs e)
        {
            if (this.ActiveControl == lvwFile)
            {
                FileCopyPaste("delete");
            }
        }

        private void MenuItemRename_Click(object sender, EventArgs e)
        {
            lvwFile.FocusedItem.BeginEdit();
        }

        private void MenuItemNewFolder_Click(object sender, EventArgs e)
        {
            string Dirname = Properties.Resources.T_NEW_FOLDER;

            ListViewItem lvi = new ListViewItem(Dirname);
            lvi.Name = Dirname;
            lvi.SubItems.AddRange(new string[] { "", "", "", "", "", "" });
            lvi.ImageKey = "folder";
            lvi.Tag = -1;

            lvwFile.Items.Add(lvi);
            lvi.BeginEdit();
            Directory.CreateDirectory(Path.Combine(current_dir, Dirname));

        }

        private void MenuItemAbout_Click(object sender, EventArgs e)
        {
            AboutBox1 dlg = new AboutBox1();
            DialogResult dlgRet = dlg.ShowDialog(this);
        }

        //----------------------------------
        // Method
        //----------------------------------
        private void FileCopyPaste(string mode)
        {
            iData = new DataObject();

            switch(mode)
            {
                case "copy":
                    if (lvwFile.SelectedItems.Count > 0)
                    {
                        string[] filenames = new string[lvwFile.SelectedItems.Count];
                        for (int i = 0; i < lvwFile.SelectedItems.Count; i++)
                        {
                            filenames[i] = Path.Combine(current_dir, lvwFile.SelectedItems[i].Text);
                        }
                        iData.SetData(DataFormats.FileDrop, filenames);
                        iData.SetData("Preferred DropEffect", copy_ms);

                        Clipboard.SetDataObject(iData);
                    }
                    break;

                case "cut":
                    if (lvwFile.SelectedItems.Count > 0)
                    {
                        string[] filenames = new string[lvwFile.SelectedItems.Count];
                        for (int i = 0; i < lvwFile.SelectedItems.Count; i++)
                        {
                            filenames[i] = Path.Combine(current_dir, lvwFile.SelectedItems[i].Text);
                        }
                        iData.SetData(DataFormats.FileDrop, filenames);
                        iData.SetData("Preferred DropEffect", cut_ms);

                        Clipboard.SetDataObject(iData);
                    }
                    break;

                case "paste":
                    iData = Clipboard.GetDataObject();

                    if (iData.GetDataPresent(DataFormats.FileDrop) != true)
                    {
                        break;
                    }

                    using (MemoryStream DropEffect = (MemoryStream)iData.GetData("Preferred DropEffect"))
                    {
                        int effect = BitConverter.ToInt32(DropEffect.ToArray(), 0);
                        string[] fileNames = (string[])iData.GetData(DataFormats.FileDrop);
                        foreach (string fileName in fileNames)
                        {
                            FileAttributes fa = File.GetAttributes(fileName);

                            if ((fa & FileAttributes.Directory) == FileAttributes.Directory)
                            {
                                if ((effect & (int)DragDropEffects.Copy) == (int)DragDropEffects.Copy)
                                {
                                    FileSystem.CopyDirectory(fileName, Path.Combine(current_dir, Path.GetFileName(fileName)),
                                                             UIOption.AllDialogs, UICancelOption.DoNothing);
                                }
                                else if ((effect & (int)DragDropEffects.Move) == (int)DragDropEffects.Move)
                                {
                                    Directory.Move(fileName, Path.Combine(current_dir, Path.GetFileName(fileName)));
                                }
                            }
                            else
                            {
                                if ((effect & (int)DragDropEffects.Copy) == (int)DragDropEffects.Copy)
                                {
                                    File.Copy(fileName, Path.Combine(current_dir, Path.GetFileName(fileName)));
                                }
                                else if ((effect & (int)DragDropEffects.Move) == (int)DragDropEffects.Move)
                                {
                                    File.Move(fileName, Path.Combine(current_dir, Path.GetFileName(fileName)));
                                }
                            }
                        }
                    }
                    break;

                case "delete":
                    if (lvwFile.SelectedItems.Count > 0)
                    {
                        if (MessageBox.Show(Properties.Resources.MSG_CONFIRM_DELETE, Properties.Resources.T_DELETE,
                                            MessageBoxButtons.YesNo, MessageBoxIcon.Warning) == DialogResult.Yes)
                        {
                            string[] filenames = new string[lvwFile.SelectedItems.Count];

                            for (int i = 0; i < lvwFile.SelectedItems.Count; i++)
                            {
                                filenames[i] = Path.Combine(current_dir, lvwFile.SelectedItems[i].Text);

                                FileAttributes fa = File.GetAttributes(filenames[i]);

                                if ((fa & FileAttributes.Directory) == FileAttributes.Directory)
                                {
                                    FileSystem.DeleteDirectory(filenames[i], UIOption.OnlyErrorDialogs, RecycleOption.SendToRecycleBin);
                                }
                                else
                                {
                                    FileSystem.DeleteFile(filenames[i], UIOption.OnlyErrorDialogs, RecycleOption.SendToRecycleBin);
                                }
                            }
                        }
                    }
                    break;
            }

        }

        private string getAOCRegistry()
        {
            string KeyName = @"SOFTWARE\Microsoft\Microsoft Games\Age of Empires II: The Conquerors Expansion\1.0";
            string GetValueName = "EXE Path";
            string location = "";

            try
            {
                Microsoft.Win32.RegistryKey Key = Microsoft.Win32.Registry.LocalMachine.OpenSubKey(KeyName);
                location = (string)Key.GetValue(GetValueName) + @"\SaveGame\";
                Key.Close();

            }
            catch (NullReferenceException)
            {
                location = @"C:\";
            }
            return location;

        }

        private void showlist(string dir)
        {

            if (Directory.Exists(dir) == false)
            {
                txtPath.Text = current_dir;
                return;
            }

            DirectoryInfo di = new DirectoryInfo(dir);
            FileSystemInfo[] fsis = di.GetFileSystemInfos();

            if (backgroundWorker1.IsBusy)
            {
                backgroundWorker1.CancelAsync();
            }
            recs.Clear();

            txtPath.Text = dir;
            current_dir = dir;
            lvwFile.Items.Clear();

            lvwFile.ListViewItemSorter = null; //sort disabled

            foreach (FileSystemInfo f in fsis)
            {
                if ((f.Attributes & FileAttributes.Directory) == FileAttributes.Directory)
                {
                    if (imgIcon.Images.IndexOfKey("folder") == -1)
                    {
                        SHFILEINFO shinfo = new SHFILEINFO();

                        IntPtr hSuccess = SHGetFileInfo(f.FullName, 0, ref shinfo, (uint)Marshal.SizeOf(shinfo),
                                                        SHGFI_ICON | SHGFI_SMALLICON);
                        if (hSuccess != IntPtr.Zero)
                        {
                            imgIcon.Images.Add("folder", Icon.FromHandle(shinfo.hIcon));
                        }
                    }
                    //set listview
                    ListViewItem lvi = lvwFile.Items.Add(f.Name);
                    lvi.Name = f.Name;
                    lvi.SubItems.AddRange(new string[] { "", "", "", "", "", "" });
                    lvi.ImageKey = "folder";
                    lvi.Tag = -1;
                }
                else if (f.Extension == ".mgx")
                {
                    if (imgIcon.Images.IndexOfKey(f.Extension) == -1)
                    {
                        SHFILEINFO shinfo = new SHFILEINFO();

                        IntPtr hSuccess = SHGetFileInfo(f.FullName, 0, ref shinfo, (uint)Marshal.SizeOf(shinfo),
                                                        SHGFI_ICON | SHGFI_SMALLICON);
                        if (hSuccess != IntPtr.Zero)
                        {
                            imgIcon.Images.Add(f.Extension, Icon.FromHandle(shinfo.hIcon));
                        }
                    }
                    //set rec class
                    Recgame rec = new Recgame();

                    FileInfo fi = new FileInfo(f.FullName);
                    rec.file_name = f.Name;
                    rec.file_size = fi.Length;
                    rec.file_attr = f.Attributes;
                    recs.Add(rec);

                    //set listview
                    ListViewItem lvi = lvwFile.Items.Add(f.Name);
                    lvi.Name = f.Name;
                    lvi.SubItems.AddRange(new string[] {"","","","","",""});
                    lvi.ImageKey = f.Extension;
                    lvi.Tag = recs.Count - 1;
                    lvi.ToolTipText = f.Name + "\n"
                                    + "Date: " + f.LastWriteTime.ToString(datefmt) + "\n"
                                    + "Size: " + ((int)(fi.Length / 1024)).ToString("N0") + " KB";
                    lvi.SubItems[5].Text = f.LastWriteTime.ToString(datefmt);

                    rec.getsha1(Path.Combine(current_dir, rec.file_name));
                    lvi.SubItems[6].Text = rec.file_hash;
                    //setLVItem(rec);
                }
            }
            fileSystemWatcher1.Path = current_dir;

            lvwFile.ListViewItemSorter = listViewItemSorter; //sort enabled

            if (!backgroundWorker1.IsBusy)
            {
                //backgroundWorker1.RunWorkerAsync(100);
            }
        }

        delegate void SetLVItemDelegate(Recgame rec);

        private void setLVItem(Recgame rec)
        {
            if (lvwFile.Items.ContainsKey(rec.file_name))
            {
                lvwFile.Items[rec.file_name].SubItems[6].Text = rec.file_hash;

                lvwFile.Items[rec.file_name].SubItems[1].Text = rec.players[rec.rec_player].name;

                lvwFile.Items[rec.file_name].SubItems[2].Text = rec.getMapName(rec.map_id);
                lvwFile.Items[rec.file_name].SubItems[3].Text = rec.vs;
                lvwFile.Items[rec.file_name].SubItems[4].Text = String.Format("{0:D}:{1:D2}:{2:D2}",
                                                                          rec.duration.Hours,
                                                                          rec.duration.Minutes,
                                                                          rec.duration.Seconds);

            }

        }
        private void setData(Recgame rec)
        {
            TextBox[] txtPlayerNames = {txtNameP1,txtNameP2,txtNameP3,txtNameP4,
                                        txtNameP5,txtNameP6,txtNameP7,txtNameP8};
            TextBox[] txtPlayerCivs = {txtCivP1,txtCivP2,txtCivP3,txtCivP4,
                                       txtCivP5,txtCivP6,txtCivP7,txtCivP8};
            TextBox[] txtPlayerTeams = {txtTeamP1,txtTeamP2,txtTeamP3,txtTeamP4,
                                        txtTeamP5,txtTeamP6,txtTeamP7,txtTeamP8};

            if (rec != null)
            {
                for (int i = 0; i < 8; i++)
                {
                    txtPlayerNames[i].Text = rec.entry_player_name[i + 1];
                    txtPlayerNames[i].ForeColor = player_color[rec.players[rec.entry_player_data[i + 1]].color];
                    txtPlayerCivs[i].Text = player_civ[rec.players[rec.entry_player_data[i + 1]].civ];
                    if (rec.entry_player_team[i + 1] != 0)
                    {
                        txtPlayerTeams[i].Text = rec.entry_player_team[i + 1].ToString();
                    }
                    else
                    {
                        txtPlayerTeams[i].Text = "";
                    }
                }

                //chat
                rec.setChat(rtxChat);
            }
            else
            {
                for (int i = 0; i < 8; i++)
                {
                    txtPlayerNames[i].Text = "";
                    txtPlayerNames[i].ForeColor = Color.Black;
                    txtPlayerCivs[i].Text = "";
                    txtPlayerTeams[i].Text = "";
                }

                //chat
                rtxChat.Clear();
            }
        }

        private void watcher_Changed(System.Object source, System.IO.FileSystemEventArgs e)
        {
            try
            {
                switch (e.ChangeType)
                {
                    case System.IO.WatcherChangeTypes.Changed:
                        break;
                    case System.IO.WatcherChangeTypes.Created:

                        FileAttributes fa = File.GetAttributes(e.FullPath);

                        if (lvwFile.Items.ContainsKey(e.Name))
                        {
                            break;
                        }

                        if ((fa & FileAttributes.Directory) == FileAttributes.Directory)
                        {
                            ListViewItem lvi = new ListViewItem(e.Name);
                            lvi.Name = e.Name;
                            lvi.SubItems.AddRange(new string[] { "", "", "", "", "", "" });
                            lvi.ImageKey = "folder";
                            lvi.Tag = -1;

                            lvwFile.Items.Add(lvi);
                        }
                        else
                        {
                            //set rec class
                            Recgame rec = new Recgame();

                            FileSystemInfo f = new FileInfo(e.FullPath);

                            FileInfo fi = new FileInfo(f.FullName);
                            rec.file_name = f.Name;
                            rec.file_size = fi.Length;
                            rec.file_attr = f.Attributes;
                            rec.getsha1(f.FullName);
                            recs.Add(rec);

                            //set listview
                            ListViewItem lvi = new ListViewItem(f.Name);
                            lvi.Name = f.Name;
                            lvi.SubItems.AddRange(new string[] { "", "", "", "", "", "" });
                            lvi.ImageKey = f.Extension;
                            lvi.Tag = recs.Count - 1;
                            lvi.ToolTipText = f.Name + "\n"
                                            + "Date: " + f.LastWriteTime.ToString(datefmt) + "\n"
                                            + "Size: " + ((int)(fi.Length / 1024)).ToString("N0") + " KB";
                            lvi.SubItems[5].Text = f.LastWriteTime.ToString(datefmt);
                            lvi.SubItems[6].Text = rec.file_hash;

                            lvwFile.Items.Add(lvi);
                        }
                        break;

                    case System.IO.WatcherChangeTypes.Deleted:
                        lvwFile.Items.RemoveAt(lvwFile.Items[e.Name].Index);
                        break;
                }
            }
            catch
            {

            }
        }
        private void watcher_Renamed(System.Object source, System.IO.RenamedEventArgs e)
        {
            try
            {
                lvwFile.Items[e.OldName].Text = e.Name;
            }
            catch
            {

            }
        }

        private void backgroundWorker1_DoWork(object sender, DoWorkEventArgs e)
        {
            int bgWorkerArg = (int)e.Argument;

            SetLVItemDelegate dlg = new SetLVItemDelegate(setLVItem);

            foreach (Recgame rec in recs)
            {
                if (backgroundWorker1.CancellationPending)
                {
                    e.Cancel = true;
                    return;
                }

                rec.getsha1(Path.Combine(current_dir, rec.file_name));
                int ret = rec.readRecgame(Path.Combine(current_dir, rec.file_name));

                if (ret == 0) //ng
                {
                    Invoke(dlg, rec);
                }
                //lvwFile.SelectedItems[0].SubItems[1].Text = rec.players[rec.rec_player].name;

                //lvwFile.SelectedItems[0].SubItems[2].Text = rec.getMapName(rec.map_id);
                //lvwFile.SelectedItems[0].SubItems[3].Text = rec.vs;
                //lvwFile.SelectedItems[0].SubItems[4].Text = string.Format("{0:D}:{1:D2}:{2:D2}",
                //                                                          rec.duration.Hours,
                //                                                          rec.duration.Minutes,
                //                                                          rec.duration.Seconds);
            }






            

        }
    }

}