using System;
using System.Collections.Generic;
using System.Text;
using System.Windows.Forms;

namespace aocrecs2
{
    public class ListViewItemComparer : System.Collections.IComparer
    {
        public enum ComparerMode
        {
            String,
            Integer,
            DateTime
        };

        private int _column;
        private SortOrder _order;
        private ComparerMode _mode;
        private string _dateformat;
        private ComparerMode[] _columnModes;

        public int Column
        {
            set
            {
                if (_column == value)
                {
                    if (_order == SortOrder.Ascending)
                        _order = SortOrder.Descending;
                    else if (_order == SortOrder.Descending)
                        _order = SortOrder.Ascending;
                }
                _column = value;
            }
            get
            {
                return _column;
            }
        }
        public SortOrder Order
        {
            set
            {
                _order = value;
            }
            get
            {
                return _order;
            }
        }
        public ComparerMode Mode
        {
            set
            {
                _mode = value;
            }
            get
            {
                return _mode;
            }
        }
        public string Dateformat
        {
            set
            {
                _dateformat = value;
            }
            get
            {
                return _dateformat;
            }
        }
        public ComparerMode[] ColumnModes
        {
            set
            {
                _columnModes = value;
            }
        }
        public ListViewItemComparer(int col, SortOrder ord, ComparerMode cmod)
        {
            _column = col;
            _order = ord;
            _mode = cmod;
        }
        public ListViewItemComparer()
        {
            _column = 0;
            _order = SortOrder.Ascending;
            _mode = ComparerMode.String;
        }

        //xy菬Ƃ̓}CiX̐A傫Ƃ̓vX̐A
        //Ƃ0Ԃ
        public int Compare(object x, object y)
        {
            int result = 0;
            //ListViewItem̎擾
            ListViewItem itemx = (ListViewItem)x;
            ListViewItem itemy = (ListViewItem)y;

            //בւ̕@
            if (_columnModes != null && _columnModes.Length > _column)
                _mode = _columnModes[_column];

            //ёւ̕@ʂɁAxyr
            switch (_mode)
            {
                case ComparerMode.String:
                    if ((int)itemx.Tag == -1 && (int)itemy.Tag != -1)
                    {
                        result = -1;
                    }
                    else if ((int)itemx.Tag != -1 && (int)itemy.Tag == -1)
                    {
                        result = 1;
                    }
                    else
                    {
                        result = string.Compare(itemx.SubItems[_column].Text, itemy.SubItems[_column].Text);
                    }
                    break;
                case ComparerMode.Integer:
                    if ((int)itemx.Tag == -1 && (int)itemy.Tag != -1)
                    {
                        result = -1;
                    }
                    else if ((int)itemx.Tag != -1 && (int)itemy.Tag == -1)
                    {
                        result = 1;
                    }
                    else
                    {
                        result = int.Parse(itemx.SubItems[_column].Text) - int.Parse(itemy.SubItems[_column].Text);
                    }
                    break;
                case ComparerMode.DateTime:
                    if ((int)itemx.Tag == -1 && (int)itemy.Tag != -1)
                    {
                        result = -1;
                    }
                    else if ((int)itemx.Tag != -1 && (int)itemy.Tag == -1)
                    {
                        result = 1;
                    }
                    else
                    {
                        DateTime date_x;
                        DateTime date_y;
                        if (itemx.SubItems[_column].Text != "")
                        {
                            date_x = DateTime.ParseExact(itemx.SubItems[_column].Text, _dateformat, null);
                        }
                        else
                        {
                            date_x = DateTime.MinValue;
                        }
                        if (itemx.SubItems[_column].Text != "")
                        {
                            date_y = DateTime.ParseExact(itemy.SubItems[_column].Text, _dateformat, null);
                        }
                        else
                        {
                            date_y = DateTime.MinValue;
                        }
                        result = DateTime.Compare(date_x, date_y);
                    }
                    break;
            }

            //~̎͌ʂ+-tɂ
            if (_order == SortOrder.Descending)
            {
                result = -result;
            }
            else if (_order == SortOrder.None)
            {
                result = 0;
            }

            //ʂԂ
            return result;
        }

    }
}
